<?php

namespace common\services\UserProducer;

use common\helpers\MeanPayment;
use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\logic\CreditHistory\CreditHistoryModel;
use common\logic\CreditHistory\CreditHistorySolver;
use common\logic\Producer\ProducerModel;
use common\logic\UserProducer\UserProducerRepository;
use common\models\Order;
use common\logic\UserProducer\UserProducerModel;

class UserProducerBuilder extends BaseService implements BuilderInterface
{
    protected CreditHistorySolver $creditHistorySolver;
    protected UserProducerRepository $userProducerRepository;

    public function __construct()
    {
        $this->creditHistorySolver = $this->loadService(CreditHistorySolver::class);
        $this->userProducerRepository = $this->loadService(UserProducerRepository::class);
    }

    public function updateCredit(CreditHistoryModel $creditHistory): void
    {
        $userProducer = $this->userProducerRepository->getOne($creditHistory->getIdUser(), $creditHistory->getIdProducer());

        if ($userProducer) {
            $oldCredit = $userProducer->getCredit();

            $this->deductCredit($userProducer, $creditHistory);
            $this->initMeanPaymentOrder($creditHistory);
            $this->sendCreditLimitReminder($userProducer, $creditHistory, $oldCredit);
        }
    }

    public function deductCredit(UserProducerModel $userProducer, CreditHistoryModel $creditHistory)
    {
        if ($this->creditHistorySolver->isTypeCredit($creditHistory)) {
            $userProducer->setCredit($userProducer->getCredit() + $creditHistory->getAmount());
        } elseif ($this->creditHistorySolver->isTypeDebit($creditHistory)) {
            $userProducer->setCredit($userProducer->getCredit() - $creditHistory->getAmount());
        }

        $userProducer->save();
    }

    public function initMeanPaymentOrder($creditHistory)
    {
        // set mean payment
        if ($creditHistory->id_order && $creditHistory->id_order > 0) {
            $order = Order::searchOne(['id' => (int)$creditHistory->id_order]);

            if ($order) {
                $paymentStatus = $order->getPaymentStatus();
                if ($paymentStatus == Order::PAYMENT_PAID
                    || $paymentStatus == Order::PAYMENT_SURPLUS) {

                    $order->mean_payment = MeanPayment::CREDIT;
                    $order->save();
                }
            }
        }
    }

    public function sendCreditLimitReminder($userProducer, $creditHistory, $oldCredit)
    {
        $userRepository = \Yii::$app->logic->getUserContainer()->getRepository();
        $producerRepository = \Yii::$app->logic->getProducerContainer()->getRepository();
        $newCredit = $userProducer->credit;

        if ($this->isCreditLimitCrossed($oldCredit, $newCredit)) {

            $user = $userRepository->getOneById($creditHistory->id_user);
            $producer = $producerRepository->getOneById($creditHistory->id_producer);

            \Yii::$app->mailer->compose(
                [
                    'html' => 'creditLimitReminder-html',
                    'text' => 'creditLimitReminder-text'
                ],
                [
                    'user' => $user,
                    'producer' => $producer,
                    'credit' => $newCredit
                ]
            )
                ->setTo($user->email)
                ->setFrom(['contact@opendistrib.net' => 'Opendistrib'])
                ->setSubject('[Opendistrib] Seuil limite de crédit dépassé')
                ->send();
        }
    }

    public function isCreditLimitCrossed($oldCredit, $newCredit)
    {
        $creditLimitReminder = ProducerModel::getConfig('credit_limit_reminder');

        return !is_null($creditLimitReminder) &&
            $oldCredit > $creditLimitReminder
            && $newCredit <= $creditLimitReminder;
    }
}